/*
**    TP 5ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <fstream>
#include <sstream>
#include <map>
#include <list>

#include <cmath>
#include <cstdlib>
#include <assert.h>

#include "skeleton.hpp"

namespace cpe
{

  const int& skeleton::parent(const unsigned int& index) const
  {
    //check index
    if(index>=v_parents.size())
    {
      std::ostringstream msg;
      msg<<"Error index "<<index<<" outside bounds of size:"<<v_parents.size();
      throw cpe::exception_skeleton(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
    //return index
    return v_parents[index];
  }
  int& skeleton::parent(const unsigned int& index)
  {
    //check index
    if(index>=v_parents.size())
    {
      std::ostringstream msg;
      msg<<"Error index "<<index<<" outside bounds of size:"<<v_parents.size();
      throw cpe::exception_skeleton(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
    //return index
    return v_parents[index];
  }
  const std::vector<int>& skeleton::get_parents() const{return v_parents;}
  std::vector<int>& skeleton::get_parents() {return v_parents;}

  const v3& skeleton::position(const unsigned int& index) const
  {
    //check index
    if(index>=v_positions.size())
    {
      std::ostringstream msg;
      msg<<"Error index "<<index<<" outside bounds of size:"<<v_positions.size();
      throw cpe::exception_skeleton(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
    //return index
    return v_positions[index];
  }
  v3& skeleton::position(const unsigned int& index)
  {
    //check index
    if(index>=v_positions.size())
    {
      std::ostringstream msg;
      msg<<"Error index "<<index<<" outside bounds of size:"<<v_positions.size();
      throw cpe::exception_skeleton(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
    //return index
    return v_positions[index];
  }
  const std::vector<v3>& skeleton::get_positions() const{return v_positions;}
  std::vector<v3>& skeleton::get_positions() {return v_positions;}

  const quaternion& skeleton::orientation(const unsigned int& index) const
  {
    //check index
    if(index>=v_orientations.size())
    {
      std::ostringstream msg;
      msg<<"Error index "<<index<<" outside bounds of size:"<<v_orientations.size();
      throw cpe::exception_skeleton(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
    //return index
    return v_orientations[index];
  }
  quaternion& skeleton::orientation(const unsigned int& index)
  {
    //check index
    if(index>=v_orientations.size())
    {
      std::ostringstream msg;
      msg<<"Error index "<<index<<" outside bounds of size:"<<v_orientations.size();
      throw cpe::exception_skeleton(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
    //return index
    return v_orientations[index];
  }
  const std::vector<quaternion>& skeleton::get_orientations() const{return v_orientations;}
  std::vector<quaternion>& skeleton::get_orientations() {return v_orientations;}

  unsigned int skeleton::number_of_bones() const{return v_parents.size();}

  skeleton::skeleton(const std::string &filename)
  {
    std::ifstream fid(filename.c_str());
    if(!fid.good())
      throw exception_skeleton("Cannot open file "+filename,__FILE__,__FUNCTION__,__LINE__);

    std::string buffer;

    //read the whole file
    while(fid.good()==true)
    {
      //read line
      std::getline(fid,buffer);

      if(buffer.size()>0&&buffer[0] != '#')
      {
        int parent;
        v3 position;
        quaternion orientation;
        std::stringstream tokens_buffer(buffer);

        tokens_buffer>>parent;
        tokens_buffer>>position.x();
        tokens_buffer>>position.y();
        tokens_buffer>>position.z();
        tokens_buffer>>orientation.x();
        tokens_buffer>>orientation.y();
        tokens_buffer>>orientation.z();
        tokens_buffer>>orientation.w();

        v_parents.push_back(parent);
        v_positions.push_back(position);
        v_orientations.push_back(orientation);
      }
    }
  }
}


