
/** TP 5ETI - CPE Lyon - 2012/2013 */


#include <cmath>

#include <sys/time.h>
#include <GL/glew.h>
#include <QString>
#include <QFont>
#include <glwidget.hpp>


#include <opengl_drawer.hpp>
#include <perlin.hpp>
#include <debug_mesh.hpp>

#include <assert.h>

#include <skeleton.hpp>

#include "scene.hpp"

using namespace cpe;

static double now_ms()
{
  static struct timeval tv_a;
  static bool first = true;
  struct timeval tv_res, tv_b;

  if (first)
  {
    gettimeofday(&tv_a, 0);
    first = false;
  }
  gettimeofday(&tv_b, 0);
  timersub(&tv_b, &tv_a, &tv_res);
  return tv_res.tv_sec * 1000. + tv_res.tv_usec / 1000.;
}


void scene::load_model()
{
  assert(p_widget!=NULL);
  assert(p_debug_mesh!=NULL);


  //*****************************************//
  // Generate grass mesh                     //
  //*****************************************//
  mesh_0.add_vertex(v3(-300,0,-300));
  mesh_0.add_vertex(v3( 300,0,-300));
  mesh_0.add_vertex(v3( 300,0, 300));
  mesh_0.add_vertex(v3(-300,0, 300));


  mesh_0.add_texture(v2( 0,  0));
  mesh_0.add_texture(v2(10,  0));
  mesh_0.add_texture(v2(10, 10));
  mesh_0.add_texture(v2( 0, 10));

  mesh_0.add_triangle(0,1,2);
  mesh_0.add_triangle(0,2,3);

  mesh_0.compute_normal();

  texture_id_0 = p_widget->bindTexture(QString("data/seamless_grass_texture.png"),GL_TEXTURE_2D);
  if(texture_id_0==0)
    std::cout<<"Erreur chargement texture data/seamless_grass_texture.png"<<std::endl;
  //*****************************************//
  // \Generate cat mesh                      //
  //*****************************************//
  mesh_cat.load_file("data/cat.obj");
  texture_cat_id = p_widget->bindTexture(QString("data/cat.png"), GL_TEXTURE_2D);
  if(texture_cat_id==0)
    std::cout<<"Erreur chargement texture data/cat.png"<<std::endl;
  cat_skeleton = new cpe::skeleton("data/cat.skeleton");
  cat_animations = new cpe::animation(cat_skeleton->number_of_bones(), "data/cat.animations");
}






void scene::draw_scene()
{
  glEnable(GL_LIGHTING);
  glEnable(GL_TEXTURE_2D);

  glColor3d(1,1,1);

  //draw mesh
  glBindTexture(GL_TEXTURE_2D,texture_id_0);
  cpe::opengl_drawer::draw(mesh_0);


  if (show_bind_pose)
  {
    glBindTexture(GL_TEXTURE_2D,texture_cat_id);
    //A COMPLETER ...


  }

  //timing and current frame
  double time = now_ms() / 1000.0;
  int start_frame = 1, end_frame = cat_animations->number_of_frames() - 2;
  time = fmod(time, (end_frame - start_frame) / speed);
  int current_frame = start_frame + time * speed;
  float local_time=fmod(time, 1/speed)*speed;





  if (show_skeleton)
  {
    glLineWidth(3.0);
    glColor3f(0,0,0);
    //A COMPLETER ...

    glLineWidth(1.0);
  }
  if (show_skinned)
  {
    glColor3f(1,1,1);
    glBindTexture(GL_TEXTURE_2D,texture_cat_id);

    //A COMPLETER ...

  }
  //draw debug information if asked in the GUI
  p_debug_mesh->draw_debug(mesh_0);
}

scene::scene() : show_bind_pose(false), show_skeleton(false), show_skinned(false)
{

  p_widget=NULL;
  p_debug_mesh=NULL;

  texture_id_0=0;
  cat_skeleton = NULL;
}


void scene::set_glwidget(glwidget *p_widget_)
{
  p_widget=p_widget_;
}
void scene::set_debug_mesh(cpe::debug_mesh *p_debug_mesh_)
{
  p_debug_mesh=p_debug_mesh_;
}

void scene::set_show_bind_pose(bool show)
{
  show_bind_pose = show;
}

void scene::set_show_skeleton(bool show)
{
  show_skeleton = show;
}

void scene::set_show_skinned(bool show)
{
  show_skinned = show;
}

void scene::set_speed(int s)
{
  speed = s;
}
